@@ -8,7 +8,7 @@ c.factor = function(...)
     for (i in seq_along(args)) if (!is.factor(args[[i]])) args[[i]] = as.factor(args[[i]])
     # The first must be factor otherwise we wouldn't be inside c.factor, its checked anyway in the line above.
     newlevels = unique(unlist(lapply(args,levels),recursive=TRUE,use.names=TRUE))
-    if (length(o <- forder(newlevels)))
+    if (length(o <- forderv(newlevels)))
         newlevels <- newlevels[o]
     nm <- names(unlist(args,recursive=TRUE,use.names=TRUE))
     ans = unlist(lapply(args, function(x) {   # Its faster this way when there are many references to the same level, which is normally the case
